from django.contrib import auth

from rest_framework import generics, status
from rest_framework.response import Response

from django_resaas.core.utils.translate import Translate
from django_resaas.models.user import User
from django_resaas.data.user.serializers.login import LoginSerializer


class ChangePasswordEmailAPIView(generics.GenericAPIView):
    def post(self, request):
        user = auth.authenticate(
            email=request.data.get('email'),
            password=request.data.get('password')
        )

        if not user:
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'A senha atual está incorreta'
                    )
                },
                status=status.HTTP_400_BAD_REQUEST
            )

        if len(request.data.get('passwordNova', '')) < 8:
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'A senha deve ter no mínimo 8 caracteres'
                    )
                },
                status=status.HTTP_400_BAD_REQUEST
            )

        user = User.objects.get(id=user.id)
        user.set_password(request.data.get('passwordNova'))
        user.save()

        return Response(
            {
                'alert_success': Translate.tdc(
                    request,
                    'Senha alterada com sucesso'
                )
            },
            status=status.HTTP_202_ACCEPTED
        )





key_value = {
    "Login efectuado com sucesso": "Login efectuado com sucesso",
    "Seleccione a Entidade": "Seleccione a Entidade",
    "Company": "Companhia",
    "Role": "Grupo",
    "Last Login":'Último login',
    "You are successfully logged in.": "Entrou com o login com sucesso.",
    "Welcome": " Bem Vindo",
    'Scaffold Wizard': 'Assistente de Scaffold',
    'Module': 'Módulo',
    'Model': 'Modelo',
    'Select module': 'Selecionar módulo',
    'Model name': 'Nome do modelo',
    'Existing models': 'Modelos existentes',
    'Fields': 'Campos',
    'Name': 'Nome',
    'Type': 'Tipo',
    'Relation app.Model': 'Relação app.Model',
    'Add field': 'Adicionar campo',
    'Preview': 'Pré-visualização',
    'Back': 'Voltar',
    'Next': 'Próximo',
    'Create': 'Criar',
    'CharField': 'Texto curto',
    'TextField': 'Texto longo',
    'IntegerField': 'Número inteiro',
    'DecimalField': 'Decimal',
    'BooleanField': 'Sim / Não',
    'DateField': 'Data',

    'ForeignKey': 'Relação (N-1)',
    'OneToOneField': 'Relação (1-1)',
    'ManyToManyField': 'Lista (N-N)',

    'FileField': 'Arquivo',
    'ImageField': 'Imagem',
    'JSONField': 'JSON / Dados livres',
    'MoneyField': 'Dinheiro'
}



import os
from django.db import models
from rest_framework import serializers
from django_resaas.core.utils.full_path import FullPath


class DynamicFieldsMixin:
    def get_fields(self):
        fields = super().get_fields()

        include = self.context.get("include_fields")
        exclude = self.context.get("exclude_fields", [])

        if include:
            fields = {k: v for k, v in fields.items() if k in include}

        for field in exclude:
            fields.pop(field, None)

        return fields


class BaseSerializer(DynamicFieldsMixin, serializers.ModelSerializer):
    """
    Base serializer:
    - dynamic fields
    - file protection
    - label/value support
    """

    label_field = None
    value_field = "id"
    permanent_fields_files = []

    # 🔹 utils

    def _get_attr(self, obj, path):
        for attr in path.split("."):
            obj = getattr(obj, attr, None)
            if obj is None:
                return None
        return obj

    def _file_representation(self, request, file, field_name):
        if not file:
            return None

        try:
            url = FullPath.url(
                request,
                file.url,
                temporary=field_name not in self.permanent_fields_files
            )

            name = os.path.basename(file.name)
            ext = os.path.splitext(name)[1].lstrip('.').lower()

            return {
                "url": url,
                "name": name,
                "ext": ext,
                "size": getattr(file, "size", None),
            }
        except Exception:
            return None

    # 🔹 core

    def to_representation(self, instance):
        data = super().to_representation(instance)
        request = self.context.get("request")

        # 🔥 label/value (simples)
        if self.label_field:
            data["label"] = self._get_attr(instance, self.label_field)
            data["value"] = getattr(instance, self.value_field, None)

        if not request:
            return data

        # 🔥 files
        for field in instance._meta.fields:
            if isinstance(field, (models.FileField, models.ImageField)):
                file = getattr(instance, field.name)
                data[field.name] = self._file_representation(request, file, field.name )

        return data


from django.conf import settings


DEFAULTS = {
    'FILE_TOKEN': {
        'KEY': 'None',
        'ENABLE_TEMPORARY': False,
        'TEMP_TTL': 300,
        'ENABLE_PERMANENT': True,
    },
    'CACHE_TIME': 300,
    'REQUIRE_FE_CREDENTIALS': False,
}


def get_setting(path, default=None):
    """
    Exemplo:
    get_setting('FILE_TOKEN.KEY')
    get_setting('REQUIRE_FE_CREDENTIALS')
    """
    config = getattr(settings, 'DJANGO_REST_AUTH', {})

    for key in path.split('.'):
        if isinstance(config, dict) and key in config:
            config = config[key]
        else:
            return default

    return config



from rest_framework import serializers

from django_resaas.models.traducao import Traducao


class TraducaoSerializer(serializers.ModelSerializer):
    class Meta:
        model = Traducao
        fields = ['id', 'chave', 'traducao', 'idioma']



import os
import re
from pathlib import Path
from django.conf import settings
from django.core.management.base import CommandError
from django_resaas.core.utils import clean_name, clean_class_name, clean_file_name
from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.models.entidade import Entidade
from django_resaas.models.modulo import Modulo
from django_resaas.models.tipo_entidade_modulo import TipoEntidadeModulo
from django_resaas.models.entidade_modulo import EntidadeModulo
from django.contrib.auth.models import Permission, Group
from django.contrib.contenttypes.models import ContentType


class ModuloScaffoldService:

    TEMPLATE = {
        "models": ["__init__.py"],
        "serializers": ["__init__.py"],
        "views": ["__init__.py"],
        "services": ["__init__.py"],
        "migrations": ["__init__.py"],
        "lang": ["ptpt.py", "enus.py"],
    }

    # =========================
    # PUBLIC
    # =========================
    @classmethod
    def create(cls, name: str):

        name = cls.clean(name)
        name = name.lower()

        base = Path(settings.BASE_DIR)
        module_path = base / name

        if module_path.exists():
            raise CommandError(f"Módulo '{name}' já existe")

        module_path.mkdir()

        # pastas
        for folder, files in cls.TEMPLATE.items():
            p = module_path / folder
            p.mkdir(parents=True)

            for f in files:
                (p / f).touch()

        p = module_path / 'templates' / name
        p.mkdir(parents=True)

        (module_path / "__init__.py").write_text(f"""default_app_config = "{clean_file_name(name)}.apps.RhConfig"\n 
        """)

        cls._create_apps(name, module_path)
        cls._create_urls(module_path)
        cls._create_sidebar(name, module_path)
        cls._create_admin(name, module_path)
        cls._add_to_settings(name)
        cls._alocate_modulo(name)

        

        return str(module_path)

    # =========================

    @staticmethod
    def clean(value):
        return re.sub(r"[^a-zA-Z0-9_]", "", value)

    # =========================

    @staticmethod
    def _create_apps(name, path):
        (path / "apps.py").write_text(f"""
import pkgutil
import importlib
from django.apps import AppConfig

class {name.title()}Config(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = '{clean_file_name(name)}'
    def ready(self):
        import {clean_file_name(name)}.views

        for _, module_name, _ in pkgutil.iter_modules({clean_file_name(name)}.views.__path__):
            importlib.import_module(f"{clean_file_name(name)}.views.{{module_name}}")
""")

    @staticmethod
    def _create_urls(path):
        (path / "urls.py").write_text("""
from django.urls import path, include
from rest_framework import routers

router = routers.DefaultRouter()

urlpatterns = [
    path("", include(router.urls)),
]
""")

    @staticmethod
    def _create_admin(name, path):
        (path / "admin.py").write_text(f"""
from django_resaas.core.base.admin import BaseAdmin
from django.contrib import admin

admin.site.site_title = '{clean_name(name)}'
admin.site.index_title = '{clean_name(name)}'

def all_fields(model):
    return [field.name for field in model._meta.fields]

""")

    @staticmethod
    def _alocate_modulo(name):
        tipo_entidade, _ = TipoEntidade.objects.get_or_create(
            nome='SaaS',
            defaults={"estado": 1}
        )

        entidade, _ = Entidade.objects.get_or_create(
            nome="Mytech",
            tipo_entidade=tipo_entidade
        )
        
        modulo, _ = Modulo.objects.get_or_create(
            nome=name,
            defaults={"estado": 1}
        )

        tipo_entidade_modulo, _ = TipoEntidadeModulo.objects.get_or_create(
            modulo=modulo,
            tipo_entidade=tipo_entidade,
            defaults={"estado": 1}
        )

        entidade_modulo, _ = EntidadeModulo.objects.get_or_create(
            modulo=modulo,
            entidade=entidade,
            defaults={"estado": 1}
        )

        
        ct, _ = ContentType.objects.get_or_create(
            app_label=name,
            model="sidebar",
        )

        admin_group, _ = Group.objects.get_or_create(name="SuperAdmin")
        perm, _ = Permission.objects.get_or_create(
            codename=f"view_{clean_file_name(name)}_dashboard",
            content_type=ct,
            defaults={"name": f"Can view {clean_file_name(name)} dashboard"},
        )

        admin_group.permissions.add(perm)


    @staticmethod
    def _create_sidebar(name, path):
        (path / "sidebar.py").write_text(f"""
MENU = "{clean_class_name(name)}"
ICON = "menu"
SUBMENUS = [
    {{
        "menu": "Dashboard",
        "icon": "dashboard",
        "role": "view_{clean_file_name(name)}_dashboard",
        "rota": "view_{clean_file_name(name)}_dashboard",
    }},
]
""")

    # =========================
    # settings MY_APPS
    # =========================




    @staticmethod
    def _add_to_settings(app_name: str):

        module_path = Path(*settings.SETTINGS_MODULE.split(".")).with_suffix(".py")
        settings_file = Path(settings.BASE_DIR) / module_path

        text = settings_file.read_text(encoding="utf-8")

        pattern = r"MY_APPS\s*=\s*\[[\s\S]*?\]"
        match = re.search(pattern, text)

        if not match:
            raise CommandError("MY_APPS não encontrado")

        block = match.group(0)

        # evitar duplicar
        if re.search(rf"['\"]{re.escape(app_name)}['\"]", block):
            return

        # adiciona antes do ]
        new_block = re.sub(
            r"\]",
            f"    '{app_name}',\n]",
            block
        )

        settings_file.write_text(text.replace(block, new_block), encoding="utf-8")


    
    @staticmethod
    def _remove_from_settings(app_name: str):

        module_path = Path(*settings.SETTINGS_MODULE.split(".")).with_suffix(".py")
        settings_file = Path(settings.BASE_DIR) / module_path

        text = settings_file.read_text(encoding="utf-8")

        pattern = r"MY_APPS\s*=\s*\[[\s\S]*?\]"
        match = re.search(pattern, text)

        if not match:
            return

        block = match.group(0)
        app_escaped = re.escape(app_name)

        # remove linha do app (qualquer posição)
        new_block = re.sub(
            rf"""
            ^[ \t]*['"]{app_escaped}['"][ \t]*,?[ \t]*(?:\#.*)?\r?\n?
            """,
            "",
            block,
            flags=re.MULTILINE | re.VERBOSE
        )

        # limpa vírgula antes do ]
        new_block = re.sub(r"\s*\]", "\n]", new_block)

        settings_file.write_text(text.replace(block, new_block), encoding="utf-8")



import base64
from datetime import datetime

import pyotp

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from django_resaas.core.utils.translate import Translate
from django_resaas.models.user import User


class generateKeyOTP:
    @staticmethod
    def returnValue(phone):
        return (
            str(phone)
            + str(datetime.date(datetime.now()))
            + settings.OTP_KEY
        )


EXPIRY_TIME = 120  # segundos


class RegisterMobileTimeAPIView(APIView):

    @staticmethod
    def get(request, phone):
        phone = '+' + str(phone).replace('+', '')

        try:
            mobile = User.objects.get(mobile=phone)
        except ObjectDoesNotExist:
            User.objects.create(mobile=phone)
            mobile = User.objects.get(mobile=phone)

        keygen = generateKeyOTP()
        key = base64.b32encode(keygen.returnValue(phone).encode())
        otp = pyotp.TOTP(key, interval=EXPIRY_TIME)

        try:
            SMS.send(
                from__='+13192205575',
                to__=phone,
                text__=Translate.tdc(
                    request,
                    'Não partilhe este código'
                ) + f':\nOTP {otp.now()}'
            )
        except Exception:
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'Erro ao enviar o OTP'
                    )
                },
                status=status.HTTP_400_BAD_REQUEST
            )

        return Response(
            {
                'alert_success': Translate.tdc(
                    request,
                    'Enviámos um OTP para o seu número'
                ),
                'data': {'otp': otp.now()}
            },
            status=status.HTTP_200_OK
        )

    @staticmethod
    def post(request, phone):
        phone = '+' + str(phone).replace('+', '')

        try:
            mobile = User.objects.get(mobile=phone)
        except ObjectDoesNotExist:
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'Utilizador não encontrado'
                    )
                },
                status=status.HTTP_404_NOT_FOUND
            )

        keygen = generateKeyOTP()
        key = base64.b32encode(keygen.returnValue(phone).encode())
        otp = pyotp.TOTP(key, interval=EXPIRY_TIME)

        if otp.verify(request.data.get('otp')):
            mobile.isVerified = True
            mobile.save()
            return Response(
                {
                    'alert_success': Translate.tdc(
                        request,
                        'Autorizado com sucesso'
                    )
                },
                status=status.HTTP_200_OK
            )

        return Response(
            {
                'alert_error': Translate.tdc(
                    request,
                    'OTP inválido ou expirado'
                )
            },
            status=status.HTTP_400_BAD_REQUEST
        )



from django_resaas.core.utils.api_response import all, ok, fail, warn, ApiResponse
from django_resaas.core.utils.clean import clean_class_name, clean_file_name, clean_name
from django_resaas.core.utils.safe_write import safe_write
from django_resaas.core.utils.bar_qr_code_64 import make_qr_b64, make_barcode_b64, png_bytes_to_b64, PDF
from django_resaas.core.utils.guess_name import guess_name
from django_resaas.core.utils.cors_allowed_origin import get_cors_origins
from django_resaas.core.utils.files import upload_path





# =========================
# Django
# =========================
from django.contrib.auth.models import Group
from django.contrib.auth.models import Permission
from django.db.models import F


# =========================
# Django REST Framework
# =========================
from rest_framework import filters
from rest_framework import status
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response


# =========================
# Local application (absolute imports)
# =========================
from django_resaas.models.user import User
from django_resaas.models.sucursal_user_group import SucursalUserGroup

from django_resaas.data.permission.serializers.permission import PermissionSerializer



class  PermissionAPIView(viewsets.ModelViewSet):
 
    search_fields = ['id','name']
    filter_backends = (filters.SearchFilter,)
    serializer_class = PermissionSerializer
    queryset = Permission.objects.annotate(content_type_model= F('content_type__model'), content_type_app= F('content_type__app_label'))
    lookup_field = "id"
    paginator = None


    def get_queryset(self):
        return self.queryset.filter().order_by('codename')

    def list(self, request, *args, **kwargs):
        self._paginator = None
        queryset = self.filter_queryset(self.get_queryset().filter().order_by('content_type__app_label', 'content_type__model', 'codename'))
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)
    
    @action(
        detail=True,
        methods=['POST'],
    )
    def addToGroup(self, request, id):
        grupo = Group.objects.get(id=request.data['id'])
        permission = Permission.objects.get(id=id)
        grupo.permissions.add(permission)
    
        per= {'id': permission.id, 'nome': permission.codename, 'nomeseparado':permission.name, 'alert_success': 'Permicao <b>' +permission.name + '</b> foi Adicionado consucesso'}
        return Response(per,status.HTTP_201_CREATED )


    @action(
        detail=True,
        methods=['POST'],
    )
    def removeFromGroup(self, request, id):
        # print(id, request.data) 
        grupo = Group.objects.get(id=request.data['id'])
        permission = Permission.objects.get(id=id)
        grupo.permissions.remove(permission)
    
        per= {'id': permission.id, 'nome': permission.codename, 'nomeseparado':permission.name, 'alert_info': 'Permicao <b>' +permission.name + '</b> foi removido consucesso'}
        return Response(per,status.HTTP_201_CREATED )


    @action(
        detail=True,
        methods=['POST'],
    )
    def addToUser(self, request, id):
        user = User.objects.get(id=request.data['user'])
        group = Group.objects.get(id=id)
        SucursalUserGroup.objects.create(sucursal_id=request.data['sucursal'], user=user, group=group)
        user.groups.add(group)
        per= {'alert_success': 'Perfil <b>' +group.name + '</b> foi Adicionado consucesso'}
        return Response(per,status.HTTP_201_CREATED )


    @action(
        detail=True,
        methods=['POST'],
    )
    def removeFromUser(self, request, id):
        user = User.objects.get(id=request.data['user'])
        group = Group.objects.get(id=id)
        SucursalUserGroup.objects.get(sucursal_id=request.data['sucursal'], user=user, group=group).delete()
        per= {'alert_success': 'Perfil <b>' +group.name + '</b> foi Removido consucesso'}
        return Response(per,status.HTTP_200_OK )


